'use strict';

exports.__esModule = true;

var _arrayFrom2 = require('./utils/arrayFrom');

var _arrayFrom3 = _interopRequireDefault(_arrayFrom2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var parseString = function parseString(str) {
  try {
    return JSON.parse(str);
  } catch (e) {
    return str;
  }
};

var convertBindingsToObject = function convertBindingsToObject(app) {
  if (!app.attributes['q-convert-bindings']) {
    return;
  }

  var commentBindings = app.innerHTML.match(/<!--\s*q-binding:\w+\[?(\d+)?\]?(\.\w+)?\s*=\s*.*?\s*-->/g);
  var elementBindings = app.querySelectorAll('[q-binding]');
  var dataObj = {};

  if (commentBindings) {
    dataObj = commentBindings.reduce(function (obj, binding) {
      var _$exec = /(\w+)\[?(\d+)?\]?(\.(\w+))?\s*=\s*(.*?)\s*-->/.exec(binding),
          name = _$exec[1],
          index = _$exec[2],
          dot = _$exec[3],
          prop = _$exec[4],
          value = _$exec[5];

      var parsedValue = parseString(value);

      if (index) {
        obj[name] = obj[name] || [];

        if (dot) {
          obj[name][index] = obj[name][index] || {};

          obj[name][index][prop] = parsedValue;
        } else {
          obj[name][index] = parsedValue;
        }
      } else if (dot) {
        obj[name] = obj[name] || {};

        obj[name][prop] = parsedValue;
      } else {
        obj[name] = parsedValue;
      }

      return obj;
    }, dataObj);
  }

  return (0, _arrayFrom3.default)(elementBindings).reduce(function (obj, binding) {
    var bindingValue = binding.attributes['q-binding'].value;
    var parsedValue = parseString(binding.textContent);

    if (bindingValue.indexOf(' as ') > 0) {
      if (bindingValue.indexOf('[') > 0) {
        var _$exec2 = /(\w+)\[(\d+)\](\.)?(.*)/.exec(bindingValue),
            name = _$exec2[1],
            index = _$exec2[2],
            dot = _$exec2[3],
            prop = _$exec2[4];

        obj[name] = obj[name] || [];

        if (dot) {
          obj[name][index] = obj[name][index] || {};

          obj[name][index][prop.split(' as ')[0].trim()] = parsedValue;
        } else {
          obj[name][index] = parsedValue;
        }
      } else {
        var _$exec3 = /(\w+)\.(\w+)\s.*/.exec(bindingValue),
            _name = _$exec3[1],
            _prop = _$exec3[2];

        obj[_name] = obj[_name] || {};
        obj[_name][_prop] = parsedValue;
      }
    } else if (binding.textContent) {
      obj[bindingValue] = parsedValue;
    }

    return obj;
  }, dataObj);
};

exports.default = convertBindingsToObject;
module.exports = exports['default'];